@@ -26,10 +26,12 @@
 #include <sys/types.h>
 
 #include "calctool.h"
+#include "unittest.h"
 #include "get.h"
 #include "display.h"
 #include "functions.h"
 #include "ui.h"
+#include "mpmath.h"
 
 time_t time();
 
@@ -51,487 +53,406 @@ struct button buttons[NKEYS] = {
 {
     KEY_0,
     "0",
-    do_number,
     NUMBER
 },
 {
     KEY_1,
     "1",
-    do_number,
     NUMBER
 },
 {
     KEY_2,
     "2",
-    do_number,
     NUMBER
 },    
 {     
     KEY_3,
     "3",
-    do_number,
     NUMBER
 },
 {
     KEY_4,
     "4",
-    do_number,
     NUMBER
 },
 {
     KEY_5,
     "5",
-    do_number,
     NUMBER
 },
 {
     KEY_6,
     "6",
-    do_number,
     NUMBER
 },
 {
     KEY_7,
     "7",
-    do_number,
     NUMBER
 },
 {
     KEY_8,
     "8",
-    do_number,
     NUMBER
 },
 {
     KEY_9,
     "9",
-    do_number,
     NUMBER
 },
 {
     KEY_A,
     "A",
-    do_number,
     NUMBER
 },
 {
     KEY_B,
     "B",
-    do_number,
     NUMBER
 },    
 {     
     KEY_C,
     "C",
-    do_number,
     NUMBER
 },
 {
     KEY_D,
     "D",
-    do_number,
     NUMBER
 },
 {
     KEY_E,
     "E",
-    do_number,
     NUMBER
 },
 {
     KEY_F,
     "F",
-    do_number,
     NUMBER
 },
 {    
     KEY_NUMERIC_POINT,
     ".",
-    do_point,
     NUMBER
 },
 {
     KEY_CALCULATE,
     NULL,
-    do_calc,
     0
 },
 {
     KEY_CLEAR,
     NULL,
-    do_clear,
     0
 },
 {
     KEY_CLEAR_ENTRY,
     NULL,
-    do_clear_entry,
     0
 },
 {
     KEY_START_BLOCK,
     "(",
-    do_paren,
     0
 },
 {
     KEY_END_BLOCK,
     ")",
-    do_paren,
     0
 },
 {
     KEY_ADD,
     "+",
-    do_calc,
     0
 },
 {
     KEY_SUBTRACT,
     "-",
-    do_calc,
     0
 },
 {
     KEY_MULTIPLY,
     "*",
-    do_calc,
     0
 },
 {
     KEY_DIVIDE,
     "/",
-    do_calc,
     0
 },
 {
     KEY_BACKSPACE,
     NULL,
-    do_backspace,
     0
 },
 {
     KEY_DELETE,
     NULL,
-    do_delete,
     0
 },
 {
     KEY_CHANGE_SIGN,
     NULL,
-    do_immed, 
     0
 },
 {
     KEY_INTEGER,
     "Int",
-    do_portion,
     FUNC
 },
 {
     KEY_FRACTION,
     "Frac",
-    do_portion,
     FUNC  
 },
 {
     KEY_PERCENTAGE,
     "%",
-    do_percent,  
     0
 },
 {
     KEY_SQUARE,
     "^2",
-    do_immed,  
     POSTFIXOP
 },
 {
     KEY_SQUARE_ROOT,
     "Sqrt",
-    do_immed, 
     FUNC
 },
 {
     KEY_RECIPROCAL,
     NULL,
-    do_immed, 
     0
 },
 {
     KEY_E_POW_X,
     "e^",
-    do_immed, 
     PREFIXOP
 },
 {
     KEY_10_POW_X,
     "10^",
-    do_immed, 
     PREFIXOP
 },       
 {
     KEY_X_POW_Y,
     "^",
-    do_calc,
     POSTFIXOP
 },
 {
     KEY_FACTORIAL,
     "!",
-    do_immed, 
     POSTFIXOP
 },
 {
     KEY_RANDOM,
     "Rand",
-    do_immed, 
     0
 },
 {
     KEY_SIN,
     "Sin",
-    do_sin,
     FUNC
 },
 {
     KEY_SINH,
     "Sinh",
-    do_sinh,
     FUNC
 },
 {
     KEY_ASIN,
     "Asin",
-    do_asin,
     FUNC
 },
 {
     KEY_ASINH,
     "Asinh",
-    do_asinh,
     FUNC
 },
 {
     KEY_COS,
     "Cos",
-    do_cos,
     FUNC
 },
 {
     KEY_COSH,
     "Cosh",
-    do_cosh,
     FUNC
 },
 {
     KEY_ACOS,
     "Acos",
-    do_acos,
     FUNC
 },
 {
     KEY_ACOSH,
     "Acosh",
-    do_acosh,
     FUNC
 },
 {
     KEY_TAN,
     "Tan",
-    do_tan,
     FUNC
 },
 {
     KEY_TANH,
     "Tanh",
-    do_tanh,
     FUNC
 },
 {
     KEY_ATAN,
     "Atan",
-    do_atan,
     FUNC
 },
 {
     KEY_TAN,
     "Atanh",
-    do_atanh,
     FUNC
 },
 {
     KEY_NATURAL_LOGARITHM,
     "Ln",
-    do_immed, 
     FUNC
 },
 {
     KEY_LOGARITHM,
     "Log",
-    do_immed, 
     FUNC
 },
 {
     KEY_LOGARITHM2,
     "Log2",
-    do_immed, 
     FUNC
 },
 {
     KEY_ABSOLUTE_VALUE,
     "Abs",
-    do_portion,
     FUNC
 },
 {
     KEY_MASK_16,
     "u16",
-    do_immed,  
     FUNC
 },            
 {
     KEY_MASK_32,
     "u32",
-    do_immed,  
     FUNC
 },
 {
     KEY_MODULUS_DIVIDE,
     " Mod ",
-    do_calc,
     0
 },
 {
     KEY_EXPONENTIAL,
     "e",
-    do_expno,
     0
 },
 {
     KEY_NOT,
     "~",
-    do_immed, 
     0
 },
 {
     KEY_OR,
     " OR ",
-    do_calc,
     0
 },
 {
     KEY_AND,
     " AND ",
-    do_calc,
     0
 },       
 {
     KEY_XOR,
     " XOR ",
-    do_calc,
     0
 },
 {
     KEY_XNOR,
     " XNOR ",
-    do_calc,
     0
 },
 {
     KEY_FINC_CTRM,
     "Ctrm",
-    do_business,
     0
 },
 {
     KEY_FINC_DDB,
     "Ddb",
-    do_business,
     0
 },
 {
     KEY_FINC_FV,
     "Fv",
-    do_business,
     0
 },
 {
     KEY_FINC_PMT,
     "Pmt",
-    do_business,
     0
 },
 {
     KEY_FINC_PV,
     "Pv",
-    do_business,
     0
 },
 {
     KEY_FINC_RATE,
     "Rate",
-    do_business,
     0
 },
 {
     KEY_FINC_SLN,
     "Sln",
-    do_business,
     0
 },
 {
     KEY_FINC_SYD,
     "Syd",
-    do_business,
     0
 },
 {
     KEY_FINC_TERM,
     "Term",
-    do_business,
     0
 },
 {
     KEY_SHIFT,
     NULL,
-    do_shift,
     0
 },
 {
     KEY_STORE,
     NULL,
-    do_sto,
     0
 },
 {
     KEY_RECALL,
     NULL,
-    do_rcl,
     0
 },
 {
     KEY_EXCHANGE,
     NULL,
-    do_exchange,
     0
 },
 {
     KEY_SET_ACCURACY,
     NULL,
-    do_accuracy,
     0
 },
 {
     KEY_CONSTANT,
     NULL,
-    do_constant,
     0
 },
 {
     KEY_FUNCTION,
     NULL,
-    do_function,
     0
 },
 };
@@ -542,22 +463,7 @@ struct button buttons[NKEYS] = {
 void
 doerr(char *errmes)
 {
-    switch (v->syntax) {
-        case NPA:
-            strncpy(v->display, errmes, MAXLINE - 1);
-            v->display[MAXLINE - 1] = '\0';
-            ui_set_error_state(TRUE);
-            ui_set_display(v->display, -1);
-            ui_beep();
-            break;
-
-        case EXPRS:
-            v->math_error = -MPMATH_ERR;
-            break;
-        
-        default:
-            assert(FALSE);
-    }
+    v->math_error = -MPMATH_ERR;
 }
 
 static void
@@ -619,7 +525,7 @@ usage(char *progname)
      * the second is the program version number.
      */
     FPRINTF(stderr, _("%s version %s\n\n"), progname, VERSION);
-    FPRINTF(stderr, _("Usage: %s: [-D] [-E] [-a accuracy] "), progname);
+    FPRINTF(stderr, _("Usage: %s: [-E] [-u] [-a accuracy] "), progname);
     FPRINTF(stderr, _("\t\t [-?] [-v] [-h]\n"));
     exit(1);
 }
@@ -635,10 +541,6 @@ get_options(int argc, char *argv[])      /* Extract command line options. */
     while (argc > 0) {
         if (argv[0][0] == '-') {
             switch (argv[0][1]) {
-                case 'D' :                   /* MP debug info. to stderr. */
-                    v->MPdebug = TRUE;
-                    break;
-
                 case 'E' :                   /* MP errors to stderr. */
                     v->MPerrors = TRUE;
                     break;
@@ -654,13 +556,9 @@ get_options(int argc, char *argv[])      /* Extract command line options. */
                         v->accuracy = DEFAULT_ACCURACY;
                     }
                     break;
-                
-                case 'n' :
-                    if (strcmp(argv[0], "-name") == 0)
-                    {
-                        INC;
-                        read_str(&v->appname, argv[0]);
-                    }
+
+                case 'u':
+                    unittest();
                     break;
 
                 case '?' :
@@ -690,7 +588,7 @@ init_constant(int n, gchar *value)
 static void
 init_state(void)
 {
-    int acc, i, n, size;
+    int acc, i, size;
 
     v->accuracy      = DEFAULT_ACCURACY;
     v->show_zeroes   = FALSE;  /* Don't show trailing zeroes. */
@@ -698,19 +596,13 @@ init_state(void)
     v->dtype         = FIX;    /* Initial number display mode. */
     v->ttype         = DEG;    /* Initial trigonometric type. */
     v->modetype      = BASIC;  /* Initial calculator mode. */
-    v->MPdebug       = FALSE;  /* No debug info by default. */
     v->MPerrors      = FALSE;               /* No error information. */
     acc              = MAX_DIGITS + 12;     /* MP internal accuracy. */
     size             = MP_SIZE;
     mpset(&acc, &size, &size);
 
-    v->error       = 0;            /* No calculator error initially. */
-    v->key_exp     = 0;            /* Not entering an exponent number. */
-    
+    v->error       = 0;            /* No calculator error initially. */    
     v->current    = KEY_CALCULATE;
-    v->shelf      = NULL;      /* No selection for shelf initially. */
-    v->noparens   = 0;         /* No unmatched brackets initially. */
-    v->numsptr    = 0;         /* Nothing on the parenthese numeric stack. */
 
     init_constant(0, "0.621");                 /* kms/hr <=> miles/hr. */
     init_constant(1, "1.4142135623");          /* square root of 2 */
@@ -723,12 +615,9 @@ init_state(void)
     init_constant(8, "0.948");                 /* Kjoules <=> BTU's. */
     init_constant(9, "0.0610");                /* cms3 <=> inches3. */
 
-    n = 0;
     for (i = 0; i < MAX_REGISTERS; i++) {
-        mpcim(&n, v->MPmvals[i]);
+        mp_set_from_integer(0, v->MPmvals[i]);
     }
-    
-    exp_clear();
 }
 
 
@@ -744,38 +633,32 @@ main(int argc, char **argv)
     textdomain(GETTEXT_PACKAGE);
 
     v->progname = argv[0];     /* Save programs name. */
-    v->appname  = NULL;
     if ((ptr = strrchr(argv[0], '/')) != NULL) {
-        read_str(&v->appname, ptr+1);
+        v->appname = strdup(ptr+1);
     } else {
-        read_str(&v->appname, argv[0]);
+        v->appname = strdup(argv[0]);
     }
     
+    srand48((long) time((time_t *) 0));   /* Seed random number generator. */    
+    
+    v->radix = get_radix();    /* Locale specific radix string. */
+    v->tsep  = get_tsep();     /* Locale specific thousands separator. */
+    v->tsep_count = get_tsep_count();
+    
     init_state();
     
     get_options(argc, argv);   /* Get command line arguments. */
     ui_init(&argc, &argv);     /* Initialise UI */
     resources_init();          /* Initialise configuration */
-
-    v->radix = get_radix();    /* Locale specific radix string. */
-    v->tsep  = get_tsep();     /* Locale specific thousands seperator. */
+    display_init(&v->display);
 
     init_text();               /* Setup text strings depending upon language. */
     read_resources();          /* Read resources from merged database. */
     ui_load();
 
-    srand48((long) time((time_t *) 0));   /* Seed random number generator. */
-
     do_clear();                /* Initialise and clear display. */
 
-    show_display(v->MPdisp_val);     /* Output in correct display mode. */
-
-    memset(&(v->h), 0, sizeof(struct exprm_state_history)); /* clear expression mode state history*/
-    struct exprm_state *e;
-    e = get_state();
-    e->clear = 1;              /* Clear initial state as if clear was pressed */
-    
-    ui_start();                    /* Display the calculator. */
+    ui_start();                /* Display the calculator. */
     
     return(0);
 }
