@@ -22,9 +22,6 @@
 #ifndef CALCTOOL_H
 #define CALCTOOL_H
 
-#include "config.h"
-#include "mp.h"
-
 #include <string.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -33,6 +30,10 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
+#include "config.h"
+#include "mp.h"
+#include "display.h"
+
 #define FCLOSE       (void) fclose     /* To make lint happy. */
 #define FPRINTF      (void) fprintf
 #define FPUTS        (void) fputs
@@ -60,13 +61,6 @@ enum num_type { ENG, FIX, SCI, MAXDISPMODES };
 /* Trigonometric types. */
 enum trig_type { DEG, GRAD, RAD, MAXTRIGMODES };
 
-/* Syntax mode. */
-enum syntax {
-    NPA = 0,      /* Non-precedence arithmetic */
-    EXPRS,        /* Expression with arithmetic precedence */
-    MAXSYNTAX
-};
-
 /* Abbreviations for the gcalctool keyboard */
 enum
 {
@@ -161,8 +155,6 @@ enum
 #undef FALSE
 #define FALSE          0
 
-#define UNDO_HISTORY_LENGTH 16  /* Arithmetic mode undo history length */
-
 #define MPMATH_ERR		    	20001
 
 enum button_flags {
@@ -175,41 +167,19 @@ enum button_flags {
 struct button {
     int id;
     char *symname;           /* Expression function name */
-    void (*func)();          /* Function to obey on button press. */
     enum button_flags flags; /* Misc flags */
 };
 
-/* Expression mode state */
-struct exprm_state {
-    int ans[MP_SIZE];      /* Previously calculated answer */
-    char *expression;      /* Expression entered by user */
-    int cursor;
-    int clear;             /* Clear command issued */
-};
-
-/* Circular list of Arithmetic Precedence Mode states*/ 
-struct exprm_state_history {
-  unsigned int begin;
-  unsigned int end;
-  unsigned int current;
-  struct exprm_state e[UNDO_HISTORY_LENGTH];  /* Expression mode state */
-};
-
 struct calcVars {                      /* Calctool variables and options. */
-    struct exprm_state_history h;      /* History of expression mode states */
-
     int current;                       /* Current button/character pressed. */
   
     char *appname;                     /* Application name for resources. */
     char *home;                        /* Pathname for users home directory. */
     char *progname;                    /* Name of this program. */
     
-    char display[MAXLINE];             /* Current calculator display. */
-    char *exp_posn;                    /* Position of the exponent sign. */
-    char fnum[MAX_LOCALIZED];          /* Scratchpad for fixed numbers. */
+    GCDisplay display;
+    
     const char *radix;                 /* Locale specific radix string. */
-    char *shelf;                       /* PUT selection shelf contents. */
-    char snum[MAX_LOCALIZED];          /* Scratchpad for scientific numbers. */
     const char *tsep;                  /* Locale specific thousands separator. */
     int tsep_count;                    /* Number of digits between separator. */
 
@@ -218,43 +188,30 @@ struct calcVars {                      /* Calctool variables and options. */
     char con_names[MAX_CONSTANTS][MAXLINE];  /* Selectable constant names. */
     int MPcon_vals[MAX_CONSTANTS][MP_SIZE];  /* Selectable constants. */
 
-    int MPdebug;                       /* If set, debug info. to stderr. */
     int MPerrors;                      /* If set, output errors to stderr. */
     int MPdisp_val[MP_SIZE];           /* Value of the current display. */
-    int MPexpr_val[MP_SIZE];           /* Value of the current expression. */
     int MPlast_input[MP_SIZE];         /* Previous number input by user. */
     int MPmvals[MAX_REGISTERS][MP_SIZE];     /* Memory register values. */
     int MPresult[MP_SIZE];             /* Current calculator total value. */
-    int MPimresult[MP_SIZE];           /* Current intermediate result. */
-    int MPtresults[3][MP_SIZE];        /* Current trigonometric results. */
 
     enum base_type base;            /* Current base: BIN, OCT, DEC or HEX. */
     enum mode_type modetype;        /* Current calculator mode. */
     enum num_type dtype;            /* Number display mode. */
     enum trig_type ttype;           /* Trig. type (deg, grad or rad). */
 
-    enum syntax syntax;             /* Calculation syntax mode */
-
     int accuracy;      /* Number of digits precision (Max 9). */
-    int cur_op;        /* Current arithmetic operation. */
+
     int error;         /* Indicates some kind of display error. */
     int math_error;    /* Math error (used in expression mode) */
-    int key_exp;       /* Set if entering exponent number. */
-    int new_input;     /* New number input since last op. */
-    int noparens;      /* Count of left brackets still to be matched. */
-    int numsptr;       /* Pointer into the parenthese numeric stack. */
-    int old_cal_value; /* Previous calculation operator. */
-    int pointed;       /* Whether a decimal point has been given. */
     int show_tsep;     /* Set if the thousands separator should be shown. */
     int show_zeroes;   /* Set if trailing zeroes should be shown. */
-    int toclear;       /* Indicates if display should be cleared. */
 };
 
 typedef struct calcVars *Vars;
 
-extern Vars v;                 /* Calctool variables and options. */
-extern int basevals[];         /* Supported arithmetic bases. */
-extern struct button buttons[];         /* Calculator button values. */
+extern Vars v;                   /* Calctool variables and options. */
+extern int basevals[];           /* Supported arithmetic bases. */
+extern struct button buttons[];  /* Calculator button values. */
 
 void doerr(char *);
 
