@@ -1,7 +1,7 @@
 
 /*  $Header$
  *
- *  Copyright (c) 1987-2007 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2008 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +22,11 @@
 #ifndef MPMATH_H
 #define MPMATH_H
 
+#include "mp.h"
+#include "calctool.h"
+
+typedef unsigned long  BOOLEAN;
+
 /* function parameters: t=target, s=source) */
 
 BOOLEAN ibool(double x);
@@ -33,7 +38,7 @@ enum trigfunc_type { sin_t, cos_t, tan_t, sinh_t, cosh_t, tanh_t,
 
 int calc_trigfunc(enum trigfunc_type type, int s1[MP_SIZE], int t1[MP_SIZE]);
 
-void mplog10(int *MPx, int *MPretval);
+void mplogn(int n, int *MPx, int *MPretval);
 void calc_and(int t[MP_SIZE], int s1[MP_SIZE], int s2[MP_SIZE]);
 void calc_or(int t[MP_SIZE], int s1[MP_SIZE], int s2[MP_SIZE]);
 void calc_xor(int t[MP_SIZE], int s1[MP_SIZE], int s2[MP_SIZE]);
@@ -59,8 +64,9 @@ void calc_rate(int t[MP_SIZE]);
 void calc_sln(int t[MP_SIZE]);
 void calc_syd(int t[MP_SIZE]);
 void calc_term(int t[MP_SIZE]);
-void calc_rshift(int s[MP_SIZE], int t[MP_SIZE], int times, enum shiftd dir);
+void calc_shift(int s[MP_SIZE], int t[MP_SIZE], int times);
 void calc_epowy(int s[MP_SIZE], int t[MP_SIZE]);
+void calc_factorial(int *, int *);
 
 /* return true if parameter is integer */
 int
@@ -72,4 +78,8 @@ is_integer(int MPnum[MP_SIZE]);
 int
 is_natural(int MPnum[MP_SIZE]);
 
+void MPstr_to_num(const char *, enum base_type, int *);
+void make_fixed(char *, int, int *, int, int, int);
+void make_number(char *, int, int *, int, int);
+
 #endif /*MPMATH_H*/
